--
-- HaweULW
--
-- @author  Manuel L. (1.LS-Modschmiede)
-- @date  25/09/08
--
-- @Umbau auf LS09 Jack85 (1.LS-Modschmiede)
-- @date 15/04/09
--
-- @Bizon Z020 edit: Burner
--

HaweULW = {};

HaweULW.UNLOADING_CLOSED = 0;
HaweULW.UNLOADING_OPENING = 1;
HaweULW.UNLOADING_OPEN = 2;
HaweULW.UNLOADING_CLOSING = 3;

function HaweULW.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function HaweULW:load(xmlFile)

    self.toggleUnloadingState = SpecializationUtil.callSpecializationsFunction("toggleUnloadingState");
    self.onStartUnloading = SpecializationUtil.callSpecializationsFunction("onStartUnloading");
    self.onEndUnloading = SpecializationUtil.callSpecializationsFunction("onEndUnloading");
    self.findTrailerRaycastCallback = Combine.findTrailerRaycastCallback;

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);

    self.pipeParticleSystem = {};
    self.pipeParticleSystemindex = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.pipeParticleSystem#index"));
    Utils.loadParticleSystem(xmlFile, self.pipeParticleSystem, "vehicle.pipeParticleSystem", self.pipeParticleSystemindex, false, nil, self.baseDirectory);

	self.unloadingDischargeEndTime = getXMLFloat(xmlFile, "vehicle.unloadingAnimation#endTime");

	local unloadingAnimRootNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.unloadingAnimation#rootNode"));
    self.unloadingAnimCharSet = 0;
    if unloadingAnimRootNode ~= nil and unloadingAnimRootNode ~= 0 then
        self.unloadingAnimCharSet = getAnimCharacterSet(unloadingAnimRootNode);
        if self.unloadingAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.unloadingAnimCharSet, getXMLString(xmlFile, "vehicle.unloadingAnimation#clip"));
            assignAnimTrackClip(self.unloadingAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.unloadingAnimCharSet, 0, false);
            self.unloadingAnimSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.unloadingAnimation#speedScale"), 1);
            self.unloadingAnimDuration = getAnimClipDuration(self.unloadingAnimCharSet, clip);
            if self.unloadingDischargeEndTime == nil then
                self.unloadingDischargeEndTime = self.unloadingAnimDuration*2.0;
            end;
        end;
    end;
	self.unloadingState = HaweULW.UNLOADING_CLOSED;

    local pipeSound = getXMLString(xmlFile, "vehicle.pipeSound#file");
    if pipeSound ~= nil and pipeSound ~= "" then
        pipeSound = Utils.getFilename(pipeSound, self.baseDirectory); 
        self.pipeSound = createSample("pipeSound");
        loadSample(self.pipeSound, pipeSound, false);
        self.pipeSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pipeSound#pitchOffset"), 1);
        self.pipeSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pipeSound#pitchScale"), 0);
        self.pipeSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pipeSound#pitchMax"), 2.0);
	  self.pipeSoundEnabled = false;
    end;

    local fillSound = getXMLString(xmlFile, "vehicle.fillSound#file");
    if fillSound ~= nil and fillSound ~= "" then
        self.fillSound = createSample("fillSound");
        loadSample(self.fillSound, fillSound, false);
        self.fillSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSound#pitchOffset"), 1);
        self.fillSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSound#pitchMax"), 2.0);
        self.fillSoundEnabled = false;
    end;
	
	self.windowGrainPlane = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.windowGrainPlane#index"));

    self.windowGrainPlaneMinY, self.windowGrainPlaneMaxY = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.windowGrainPlane#minMaxY"));
    if self.windowGrainPlaneMinY == nil or self.windowGrainPlaneMaxY == nil then
        local windowGrainAnimCurve = AnimCurve:new(linearInterpolator4);
        local keyI = 0;
        while true do
            local key = string.format("vehicle.windowGrainPlane.key(%d)", keyI);
            local t = getXMLFloat(xmlFile, key.."#time");
            local yValue = getXMLFloat(xmlFile, key.."#y");
            local scaleX,scaleY,scaleZ = Utils.getVectorFromString(getXMLString(xmlFile, key.."#scale"));
            if y == nil or scaleX == nil or scaleY == nil or scaleZ == nil then
                break;
            end;
            windowGrainAnimCurve:addKeyframe({x=scaleX, y=scaleY, z=scaleZ, w=yValue, time = t});
            keyI = keyI +1;
        end;
        if keyI > 0 then
            self.windowGrainAnimCurve = windowGrainAnimCurve;
        end;
        self.windowGrainPlaneMinY = 0;
        self.windowGrainPlaneMaxY = 0;
    end;

	self.trailerPosition = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.trailerPosition#index"));
	self.grainUnloading = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.grainUnloading"), 10);

	self.isExpanded = false;
	self.pipeParticleActivated = false;
	self.isActive = false;
	self.bcActive = false;
	self.unloadingSpeed = 1;
end;

function HaweULW:delete()
end;

function HaweULW:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HaweULW:keyEvent(unicode, sym, modifier, isDown)
end;

function HaweULW:update(dt)
	if self.attacherVehicle ~= nil then
		if self:getIsActive() then
			if InputBinding.hasEvent(InputBinding.EMPTY_GRAIN) then
				self:toggleUnloadingState();
			end;
			if self.trailerFound ~= 0 then
				if InputBinding.hasEvent(InputBinding.TOGGLEUNLOADGRAIN) then
					self.activeUnloading = not self.activeUnloading;
					self.pipeParticleActivated = not self.pipeParticleActivated;
				end;
			end;
		end;
	end;

	if self.unloadingState == HaweULW.UNLOADING_OPENING then 
        if self.pipeSound ~= nil and not self.pipeSoundEnabled then
            setSamplePitch(self.pipeSound, self.pipeSoundPitchOffset);
            playSample(self.pipeSound, 0, 1, 0);
            self.pipeSoundEnabled = true;
        end;
   	end;

	if self.unloadingState == HaweULW.UNLOADING_OPEN then 
        if self.pipeSound ~= nil and self.pipeSoundEnabled then
            stopSample(self.pipeSound);
            self.pipeSoundEnabled = false;
        end;
   	end;

	if self.unloadingState == HaweULW.UNLOADING_CLOSING then 
        if self.pipeSound ~= nil and not self.pipeSoundEnabled then
            setSamplePitch(self.pipeSound, self.pipeSoundPitchOffset);
            playSample(self.pipeSound, 0, 1, 0);
            self.pipeSoundEnabled = true;
        end;
   	end;

	if self.unloadingState == HaweULW.UNLOADING_CLOSED then 
        if self.pipeSound ~= nil and self.pipeSoundEnabled then
            stopSample(self.pipeSound);
            self.pipeSoundEnabled = false;
        end;
   	end;

	if self.activeUnloading and self.pipeParticleActivated then
        if not self.fillSoundEnabled and self.fillLevel > 0 and self.fillSound ~= nil then
            playSample(self.fillSound, 0, self.fillSoundVolume, 0);
            self.fillSoundEnabled = true;   
        end;
	else
        if self.fillSoundEnabled then
            stopSample(self.fillSound);
            self.fillSoundEnabled = false;   
        end;
	
	end;
	
	if self.unloadingState == HaweULW.UNLOADING_OPENING or self.unloadingState == HaweULW.UNLOADING_OPEN then
        if self.unloadingState == HaweULW.UNLOADING_OPENING then
            if getAnimTrackTime(self.unloadingAnimCharSet, 0) > self.unloadingAnimDuration then
                self.unloadingState = HaweULW.UNLOADING_OPEN;
            end;
        end;
    elseif self.unloadingState == HaweULW.UNLOADING_CLOSING then
        if getAnimTrackTime(self.unloadingAnimCharSet, 0) < 0.0 then
            self.unloadingState = HaweULW.UNLOADING_CLOSED;
        end;
    end;
	
	if self:getIsActive() then	
		self.trailerFound = 0;
		local x,y,z = getWorldTranslation(self.trailerPosition);
		raycastAll(x, y, z, 0, -1, 0, "findTrailerRaycastCallback", 10, self);
		
		if self.attacherVehicle ~= nil then
			if self.trailerFound ~= 0 and self.attacherVehicle.isAutoActivated then
				self.activeUnloading = true
			end
		end
		
		self.currentTipTrigger = nil;			
		self.lastUnloadingTrailer = nil;
		self.pipeParticleActivated = false;
		
		if self.activeUnloading then
			if self.unloadingState == HaweULW.UNLOADING_OPEN and self.fillLevel > 0 then
				self.pipeParticleActivated = true;
				local trailer = g_currentMission.nodeToVehicle[self.trailerFound];
				self.lastUnloadingTrailer = trailer;
				if self.trailerFound == 0 or trailer == nil or not trailer:allowFillType(self.currentFillType, true) or not trailer.allowFillFromAir then
					self.pipeParticleActivated = false;
					self.activeUnloading = false;
				else
					local deltaLevel = self.unloadingSpeed*dt/10.0;
					deltaLevel = math.min(deltaLevel, trailer.capacity - trailer.fillLevel);

					self.fillLevel = self.fillLevel-deltaLevel;
					if self.fillLevel <= 0.0 then
						deltaLevel = deltaLevel+self.fillLevel;
						self.fillLevel = 0.0;
						self.pipeParticleActivated = false;
						self.activeUnloading = false;
					end;
					if deltaLevel == 0 then
						self.pipeParticleActivated = false;
						self.activeUnloading = false;
					end;
					self:setFillLevel(self.fillLevel, self.currentFillType);
					trailer:setFillLevel(trailer.fillLevel+deltaLevel, self.currentFillType);
				end;
			end;
		end;
	end;

	if self.activeUnloading and self.pipeParticleActivated then
		Utils.setEmittingState(self.pipeParticleSystem, true);
	else
		Utils.setEmittingState(self.pipeParticleSystem, false);
	end;
end;

function HaweULW:draw()
	if self.attacherVehicle ~= nil then
		if self.unloadingState == HaweULW.UNLOADING_OPEN or self.unloadingState == HaweULW.UNLOADING_OPENING then
			g_currentMission:addHelpButtonText("Raise Grain Pipe", InputBinding.EMPTY_GRAIN);
		else
			g_currentMission:addHelpButtonText("Lower Grain Pipe", InputBinding.EMPTY_GRAIN);
		end;
		if self.unloadingState == HaweULW.UNLOADING_OPEN and self.trailerFound ~= 0 and self.fillLevel > 0 then
			if self.activeUnloading and self.pipeParticleActivated then
				g_currentMission:addHelpButtonText("Stop Grain auger", InputBinding.TOGGLEUNLOADGRAIN);
			else
				g_currentMission:addHelpButtonText("Start Grain auger", InputBinding.TOGGLEUNLOADGRAIN);
			end;
		end;
	end;
end;

function HaweULW:onDetach()
end;

function HaweULW:setFillLevel(fillLevel, fillType)
	Trailer.setFillLevel(self, fillLevel, fillType);
    if self.windowGrainPlane ~= nil then
        local yTranslation;
        if self.windowGrainAnimCurve then
            local scaleX, scaleY, scaleZ , yTrans = self.windowGrainAnimCurve:get(self.fillLevel/self.capacity);
            yTranslation = yTrans;
            setScale(self.windowGrainPlane, scaleX, scaleY, scaleZ);
        else
            local m = (self.windowGrainPlaneMaxY - self.windowGrainPlaneMinY) / self.capacity;
            yTranslation = m*self.fillLevel + self.windowGrainPlaneMinY;
        end;
            local xPos, yPos, zPos = getTranslation(self.windowGrainPlane);
        setTranslation(self.windowGrainPlane, xPos, yTranslation, zPos);
        setVisibility(self.windowGrainPlane, self.fillLevel ~= 0);
    end;

end;

function HaweULW:findTrailerRaycastCallback(transformId, x, y, z, distance)
    if getUserAttribute(transformId, "vehicleType") == 2 and transformId ~= self.rootNode then
        self.trailerFound = transformId;
    end;
    return false;
end;

function HaweULW:toggleUnloadingState()
    if self.unloadingState == 0 then
        self:onStartUnloading();
    else
        self:onEndUnloading();
    end;
end;

function HaweULW:onStartUnloading()
    if self.unloadingAnimCharSet ~= 0 then
        if getAnimTrackTime(self.unloadingAnimCharSet, 0) < 0.0 then
            setAnimTrackTime(self.unloadingAnimCharSet, 0, 0.0);
        end;
        setAnimTrackSpeedScale(self.unloadingAnimCharSet, 0, self.unloadingAnimSpeedScale);
        enableAnimTrack(self.unloadingAnimCharSet, 0);
    end;
    self.unloadingState = HaweULW.UNLOADING_OPENING;
end;

function HaweULW:onEndUnloading()
    if self.unloadingAnimCharSet ~= 0 then
        if getAnimTrackTime(self.unloadingAnimCharSet, 0) > self.unloadingAnimDuration then
            setAnimTrackTime(self.unloadingAnimCharSet, 0, self.unloadingAnimDuration);
        end;
        setAnimTrackSpeedScale(self.unloadingAnimCharSet, 0, -self.unloadingAnimSpeedScale);
        enableAnimTrack(self.unloadingAnimCharSet, 0);
    end;
    self.unloadingState = HaweULW.UNLOADING_CLOSING;
end;

function HaweULW:toggleTipState()
end;